import { useCallback, useMemo, useRef, useState } from 'react';
import { useWindowDimensions } from 'react-native';
import { interpolate, useAnimatedScrollHandler, useAnimatedStyle, useDerivedValue, useSharedValue, } from 'react-native-reanimated';
import { useSafeAreaInsets } from 'react-native-safe-area-context';
import styles from '../styles';
import useScrollSync from './useScrollSync';
const TAB_BAR_HEIGHT = 48;
const HEADER_HEIGHT = 48;
const OVERLAY_VISIBILITY_OFFSET = 32;
const useParallaxHeaderWithTab = (tabs) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const { top, bottom } = useSafeAreaInsets();
    const { height: screenHeight } = useWindowDimensions();
    //@todo: Tabs should be rendered dynamically
    const firstTabRef = useRef(null);
    const secondTabRef = useRef(null);
    const thirdTabRef = useRef(null);
    const fourthTabRef = useRef(null);
    const [tabIndex, setTabIndex] = useState(0);
    const [headerHeight, setHeaderHeight] = useState(0);
    const defaultHeaderHeight = top + HEADER_HEIGHT;
    const headerConfig = useMemo(() => ({
        heightCollapsed: defaultHeaderHeight,
        heightExpanded: headerHeight,
    }), [defaultHeaderHeight, headerHeight]);
    const { heightCollapsed, heightExpanded } = headerConfig;
    const headerDiff = heightExpanded - heightCollapsed;
    const rendered = headerHeight >= 0;
    const handleHeaderLayout = useCallback((event) => setHeaderHeight(event.nativeEvent.layout.height), []);
    const firstTabScrollValue = useSharedValue(0);
    const firstTabScrollHandler = useAnimatedScrollHandler((event) => (firstTabScrollValue.value = event.contentOffset.y));
    const secondTabScrollValue = useSharedValue(0);
    const secondTabScrollHandler = useAnimatedScrollHandler((event) => (secondTabScrollValue.value = event.contentOffset.y));
    const thirdTabScrollValue = useSharedValue(0);
    const thirdTabScrollHandler = useAnimatedScrollHandler((event) => (thirdTabScrollValue.value = event.contentOffset.y));
    const fourthTabScrollValue = useSharedValue(0);
    const fourthTabScrollHandler = useAnimatedScrollHandler((event) => (fourthTabScrollValue.value = event.contentOffset.y));
    const scrollPairs = useMemo(() => [
        { list: firstTabRef, position: firstTabScrollValue },
        { list: secondTabRef, position: secondTabScrollValue },
        { list: thirdTabRef, position: thirdTabScrollValue },
        { list: fourthTabRef, position: fourthTabScrollValue },
    ], [
        firstTabScrollValue,
        fourthTabScrollValue,
        secondTabScrollValue,
        thirdTabScrollValue,
    ]);
    const { sync } = useScrollSync(scrollPairs, headerConfig);
    const сurrentScrollValue = useDerivedValue(() => tabIndex === 0
        ? firstTabScrollValue.value
        : tabIndex === 1
            ? secondTabScrollValue.value
            : tabIndex === 2
                ? thirdTabScrollValue.value
                : fourthTabScrollValue.value, [
        tabIndex,
        firstTabScrollValue,
        secondTabScrollValue,
        thirdTabScrollValue,
        fourthTabScrollValue,
    ]);
    const translateY = useDerivedValue(() => -Math.min(сurrentScrollValue.value, headerDiff));
    const tabBarAnimatedStyle = useAnimatedStyle(() => ({
        transform: [{ translateY: translateY.value }],
    }));
    const headerAnimatedStyle = useAnimatedStyle(() => ({
        transform: [{ translateY: translateY.value }],
        opacity: interpolate(translateY.value, [-headerDiff, 0], [0, 1]),
    }));
    const contentContainerStyle = useMemo(() => ({
        paddingTop: rendered ? headerHeight + TAB_BAR_HEIGHT : 0,
        paddingBottom: bottom,
        minHeight: screenHeight + headerDiff,
    }), [rendered, headerHeight, bottom, screenHeight, headerDiff]);
    const sharedProps = useMemo(() => ({
        contentContainerStyle,
        onMomentumScrollEnd: sync,
        onScrollEndDrag: sync,
        scrollEventThrottle: 16,
        scrollIndicatorInsets: { top: heightExpanded },
    }), [contentContainerStyle, sync, heightExpanded]);
    const tabBarStyle = useMemo(() => [
        rendered ? styles.tabBarContainer : undefined,
        { top: rendered ? headerHeight : undefined },
        tabBarAnimatedStyle,
    ], [rendered, headerHeight, tabBarAnimatedStyle]);
    const headerContainerStyle = useMemo(() => [
        rendered ? styles.headerContainer : undefined,
        { paddingTop: top },
        headerAnimatedStyle,
    ], [rendered, top, headerAnimatedStyle]);
    const collapsedOverlayAnimatedStyle = useAnimatedStyle(() => ({
        opacity: interpolate(translateY.value, [-headerDiff, OVERLAY_VISIBILITY_OFFSET - headerDiff, 0], [1, 0, 0]),
    }));
    const collapsedOverlayStyle = useMemo(() => [
        styles.collapsedOvarlay,
        collapsedOverlayAnimatedStyle,
        { height: heightCollapsed, paddingTop: top },
    ], [collapsedOverlayAnimatedStyle, heightCollapsed, top]);
    // array contains tab references, scrollEvent, flatList data, renderItem as renderTabContent, otherProps as renderFlatListProps
    const tabsHandlers = [
        {
            tabRef: firstTabRef,
            tabScrollHandler: firstTabScrollHandler,
            data: (_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.data,
            renderTabContent: (_b = tabs[0]) === null || _b === void 0 ? void 0 : _b.renderTabContent,
            renderFlatListProps: (_c = tabs[0]) === null || _c === void 0 ? void 0 : _c.renderFlatListProps,
        },
        {
            tabRef: secondTabRef,
            tabScrollHandler: secondTabScrollHandler,
            data: (_d = tabs[1]) === null || _d === void 0 ? void 0 : _d.data,
            renderTabContent: (_e = tabs[1]) === null || _e === void 0 ? void 0 : _e.renderTabContent,
            renderFlatListProps: (_f = tabs[1]) === null || _f === void 0 ? void 0 : _f.renderFlatListProps,
        },
        {
            tabRef: thirdTabRef,
            tabScrollHandler: thirdTabScrollHandler,
            data: (_g = tabs[2]) === null || _g === void 0 ? void 0 : _g.data,
            renderTabContent: (_h = tabs[2]) === null || _h === void 0 ? void 0 : _h.renderTabContent,
            renderFlatListProps: (_j = tabs[2]) === null || _j === void 0 ? void 0 : _j.renderFlatListProps,
        },
        {
            tabRef: fourthTabRef,
            tabScrollHandler: fourthTabScrollHandler,
            data: (_k = tabs[3]) === null || _k === void 0 ? void 0 : _k.data,
            renderTabContent: (_l = tabs[3]) === null || _l === void 0 ? void 0 : _l.renderTabContent,
            renderFlatListProps: (_m = tabs[3]) === null || _m === void 0 ? void 0 : _m.renderFlatListProps,
        },
    ];
    return {
        tabsHandlers,
        tabIndex,
        handleHeaderLayout,
        sharedProps,
        setTabIndex,
        tabBarStyle,
        headerContainerStyle,
        collapsedOverlayStyle,
    };
};
export default useParallaxHeaderWithTab;
//# sourceMappingURL=useParallaxHeaderWithTab.js.map